; QB Copyright (C)2001 Andrew Davie
; see http://homepages.ihug.co.nz/~aaronat/qb.html

; DO NOT TRUST ABSOLUTELY THE COMMENTS IN THIS CODE!
; MOST OF THEM ARE OK.  SOME ARE MISLEADING :)

; History: original PUSH demo posted to Stella list May 1998
; This demonstrated multiple-colour playfields and VERY rudimentary background animation

; Extended January/February 2001 to display a 4x4 tiled grid, each tile independantly movable.



    processor 6502
    include vcs.h

;------------------------------------------------------------------------------
; Compilation switches...

COMPILE_ILLEGALOPCODES  = 1             ;             set to 0 for NO illegal opcodes, 1 for YES illegal opcodes
        ;\--> Users of Stella (MAC users) need to set the above to 0 (for now, anyway)

NTSC            = 0
PAL             = 1

COMPILE_VERSION         = NTSC           ; NTSC/PAL    switches display and colours


; Possible savings - set the following to 0 to disable (and save memory!)

PLAYER_RISES            = 0 ;1             ; [ 10 bytes] player has rising frame defined [20 if hair]
PLAYER_HAS_HAIR         = 0             ; [ 12 bytes] gives player interlaced hair [22 if rising]
PLAYER_WEARS_HAT        = 0             ; [     free] dodgy colour change for hat
PROTECT_SCORELINE       = 1             ; [102 bytes] protect the sprites when over score
ATTRACT_MODE            = 1             ; [ 13 bytes] cycle colours during demo
SLOW_AUTOMOVE           = 0             ; [ 34 bytes] auto-moved cubes move slowly

; Gameplay tweaking...

TIME_PAUSE_AFTER_JUMP   = 5             ; max 31

PLAYERCOLOUR            = COLUP0        ; COLUP0 or COLUBK (lets you see colour timing)
PLAYER0SHAPE            = GRP0          ; PLAYER0SHAPE or COLUBK "
PLAYER1SHAPE            = GRP1          ; PLAYER1SHAPE or COLUBK "

;------------------------------------------------------------------------------
; THINGS YOU DON'T CHANGE...

 
CUBEXY          = 5                     ; size of cube + border
PFSIZE          = 3*(4*CUBEXY-1)        ; 3 bytes / line by n lines
PFC             = 19                    ; actual rows of playfield

NPLAYERS        = 2                     ; DO NOT CHANGE

SPRITE_Y_ADJUST = 24                     ; adjust vertical position of sprites (pixels down from top of cube)
SPRITE_X_ADJUST = 22                    ; horizontal pixel adjust to align with grid

GUARANTEED0READ = MTA_ZERO                  ; ANY zero page locations with 0,0 in consecutive bytes


NUMBEROFCUBES   = 6                     ; cubes which move
 ;\---> see dependencies :
 ;       - PlayerCube assumes <= 8
 ;       - uses 2n bytes RAM
 ;      NOT MUCH POINT >8 CUBES, AS IT GETS CLUTTERED/EASY!


SPRITE_HEIGHT   = 11
 

;------------------------------------------------------------------------------
; Generic equates

; bits
D0              = 1
D1              = 2
D2              = 4
D3              = 8
D4              = 16
D5              = 32
D6              = 64
D7              = 128

NO_RISE         = %10000000             ; marks sprite definition as NOT rising
BLANK_LINE      = 0                     ; header for all sprite frames

;------------------------------------------------------------------------------
; variables

    SEG.U vars
    org $80

    ; only 128 bytes... including the stack!
    ; let the assembler calculate the addresses

;ZERO            ds 2                    ; ZERO! (2 bytes, as MTargetArea indexing is 0,y and 1,y)
timer           ds 2                    ; generic 1-per-frame timer + screen rate (byte 2)
PlayerCounter   ds NPLAYERS             ; generic counter for event timing
                ; mmmccccc
                ; m = creature mode
                ; c = creature counter
        ;\--> mode indicates what to do
        ;     counter is used for THAT mode only

        ;\--> usage/mode definitions
MODEMASK        = %11100000
MODE_NORMAL     = %00000000                     ; doing nothing special
MODE_RISE       = %00100000                     ; rising up out of cubes
MODE_SINK       = %01000000                     ; sinking into a cube
MODE_LURK       = %01100000                     ; lurking, waiting for a cube
MODE_JUMP       = %10000000                     ; jumping
MODE_LOCK       = %10100000                     ; locked, pending screen change
MODE_DELAY      = %11000000                     ; just delays

CubeX           ds NUMBEROFCUBES        ; d0-d4 = X(0-31), high 3 bits unused.

        IF SLOW_AUTOMOVE
                ; vs-xxxxx
                ; v = valid cube (1 = invalid)
                ; x = x coordinate (0-31).  16+ is off board
                ; s = slow movement
        ;\-->usage
MOVE_SLOW               = %01000000
MOVE_SLOW_MASK          = 7
MOVE_SLOW_ENABLE        = 4

        ELSE
                ; v--xxxxx
                ; v = valid cube (1 = invalid)
                ; x = x coordinate (0-31).  16+ is off board
                ; - = unused

        ENDIF



CubeYDir        ds NUMBEROFCUBES
                ; sdddyyyy
                ; y = Y coordinate (0-15)
                ; d = direction:
                ;       000     up
                ;       001     right
                ;       010     down
                ;       011     left
                ;       100     upright
                ;       101     downright
                ;       110     downleft
                ;       111     upleft
                ; s = stationary (1 = stopped, 0 = moving)


rnd             ds 2                    ; random #

PlayerFlags     ds NPLAYERS             ; generic flags
                ; dddmfffr
                ; d = direction of moving/facing. x00=up, x01=right, x10=down, x11=left
                        ;\-->being extended for diagonals... in progress
                ; f = frame to display for animation
                ; r = use creature counter as rise/fall display offset for sprites
                ; m = mirror sprite (left-facing)
        ;\--> usage definitions
FLAG_RISE       = D0                    ; use counter as rise/fall
FLAG_MIRROR     = D4

PlayerX         ds NPLAYERS             ; x coordinate
PlayerY         ds NPLAYERS             ; y offset for jumping, etc.

PlayerCube      ds NPLAYERS
                ; values 0-NUMBEROFCUBES-1
                ; Let's assume NUMBEROFCUBES <= 8  (OK, so that's only 3 bits required)
                ; xddddiii
                ; i, d0-d2 = cube index
                ; d, d3-d6 = sprite/creature ID (used for graphic selection) (16 objects)
                ; x, d7    = unused

TARGETSIZE = 5
TargetDisplay   ds 2*TARGETSIZE         ; visible target display (flashes/blinks, etc)
                ; TargetDisplay hold the graphics for the entire right-side of the screen
                ; The first 4 bytes in each are the target area, which is flashed/animated
                ; The last byte is (all of!) the sliding cube feeding onto the grid.

Grid            ds 4                    ; 4 rows.  2 bits/tile for info
                ; GRID: organised into 4 rows, 1 byte/row
                ;       each byte has 2 bits/tile, ordered target/ownership
                ;       target = a cube 'belongs' here
                ;       ownership = a cube IS here


playfield       ds 3 * 19
        ; displays on the screen horizontally, but organised in COLUMNS, each PFC bytes deep
        ; there are 3 columns.  Organisation this way saved nearly 50 bytes of code!

level           ds 1                    ; game level (increases difficulty)
        ; mmmlllll
        ; mmm = # of lives (0-7)
        ; lllll = level (0-31)

Cubes           ds 1                    ; number of cubes on this board
        ; dsxffccc
        ;  = unused
        ; s = grid complete, flash!
        ; d = demo mode
        ; f = flash score/related
        ; c = cube count
        ;\-->equates...
CUBES_DEMO_MODE equ %10000000           ; game is in demo mode
CUBES_COMPLETED equ %01000000           ; grid has completed


BackColour      ds 1                    ; background colour (mainly debugging)

PlayerScore     ds 3                    ; player's score (3 BCD digits)

        ; /DODGY ON
        ; This overlay variable is used for the overlay variables.  That's OK.
        ; However, it is positioned at the END of the variables so, if on the off chance we're overlapping
        ; stack space and variable, it is LIKELY that that won't be a problem, as the temp variables
        ; (especially the latter ones) are only used in rare occasions.

        ; FOR SAFETY, DO NOT USE THIS AREA DIRECTLY (ie: NEVER reference 'Overlay' in the code)
        ; ADD AN OVERLAY FOR EACH ROUTINE'S USE, SO CLASHES CAN BE EASILY CHECKED


Overlay         ds 17                   ;--> OVERLAYS

        ; /DODGY OFF

;------------------------------------------------------------------------------
; OVERLAYS!
; These variables are overlays, and should be managed with care
; That is, variables are ALREADY DEFINED, and we're reusing RAM for other purposes

; EACH OF THESE ARE VARIABLES (TEMPORARY) USED BY ONE ROUTINE (AND IT'S SUBROUTINES)
; THAT IS, LOCAL VARIABLES.  USE 'EM FREELY, THEY COST NOTHING

; TOTAL SPACE USED BY ANY OVERLAY GROUP SHOULD BE <= SIZE OF 'Overlay'

;------------------------------------------------------------------------------
; MTarget(Area) overlays...
        
        org Overlay ;[17/17]

SpriteEnd       ds 2
Sprite          ds 4                    ; -> sprite graphic shape

; DrawScreen overlays...

DS_Sprite       ds 2
DS_temp         ds 1
MT_Overlay      ds 8                    ;--> OVERLAYS

; SUBSETS of DrawScreen
; PositionSprites overlays...

        org MT_Overlay ;[1/8]

PS_temp         ds 1

; MTargetAreaBorder overlays...

        org MT_Overlay ;[8/8]

MT_TargetMask   ds 1                    ; mask for target area border (SHARED)

; MTargetArea overlays

MTA_TargetL     ds 1                    ; calculated target-area display bytes
MTA_TargetR     ds 1
MTA_TargBorder  ds 1

MTA_ZERO        ds 2                    ; BRILLIANT!  gives us our two required 0-reads
MTA_31          ds 2

        org MT_Overlay ;[1/8]

SL_temp         ds 1


;------------------------------------------------------------------------------
; ScoreLine overlays...
; OVERLAP WITH DRAWSCREEN... BE *VERY* CAREFUL NOT TO BRACKET USAGE
; ACROSS SCORELINE!

        org Overlay ;[15/17]

stkp            ds 1
sreg            ds 1
loop            ds 1

S0              ds 2                    ; used for addressing digits of score
S1              ds 2
S2              ds 2
S3              ds 2
S4              ds 2
S5              ds 2



;------------------------------------------------------------------------------
; DoAI overlays... 

        org Overlay ;[11/15]

AI_Vector       ds 2
AI_Player       ds 1
AI_Overlay      ds 4                    ;--> OVERLAYS

; SUBSETS of DOAI
; CountDown overlays...

CD_TimerMask    ds 1

; AI_Jump overlays...
AIJ_YSlower     ds 1

; AI_Lurk overlays...
AIL_LurkerCube  ds 1

; NextLevel overlays...
NL_temp         ds 1

; SUB-OVERLAYS OF DoAI...(!)
; OVERLAYS INSIDE OVERLAYS

        org AI_Overlay ;[4/4]
; CheckMove (sub)overlays...
CM_CubeIndex    ds 1
CM_Direction    ds 1
CM_Column       ds 1
CM_scratch      ds 1

        org AI_Overlay ;[4/4]
; CheckJump (sub)overlays...
CJ_CubeIndex    ds 1
CJ_Direction    ds 1
CJ_Overlay      ds 2                    ; --> OVERLAYS

        org CJ_Overlay ;[2/2]
CJ_NewX         ds 1
CJ_NewY         ds 1

        org CJ_Overlay ;[1/2]
CJ_CubeNumber   ds 1

;------------------------------------------------------------------------------
; InitCubes overlays...

        org Overlay ;[8/15]

IC_FedCubeIndex ds 1

; SUBSETS OF InitCubes...
; CreateTableaux overlays...
CT_Cubes        ds 1
CT_index        ds 1
CT_coord        ds 1

; FeedCube overlays...
FC_FedCubeIndex ds 1
FC_BlankSpot    ds 1
FC_CubeX        ds 1
FC_CubeY        ds 1
FC_temp2        ds 1
FC_coord        ds 1

;------------------------------------------------------------------------------
; MoveCube overlay...

        org Overlay ;[6/15]

MC_temp         ds 1
MC_DirOnly      ds 1
MC_HasMoved     ds 1

; SUBSETS OF MoveCube...
; DrawCube overlay...

DC_scratch      ds 1
DC_CubeLine     ds 1
DC_saveX        ds 1

;------------------------------------------------------------------------------

        ; add more overlays here, as required...


    SEG code
    org $F000
; eek! just 4k (-vectors)


;------------------------------------------------------------------------------

PreDrawn3
 ;---> merges with above 


        REPEAT 17
        .byte 0
        REPEND

    .byte %10000000
    .byte %11000000
    .byte %11100000

;\--> merges with next table... don't move 'em!        

PreDrawn0

    ;     pf1 -->   pf2 <--   pf0 <--   pf1 -->   pf2 <--
    ;     ok        flip      flip xxxx ok        flip
    .byte %11110000     ; 0
    .byte %01111000; 1
    .byte %00111100; 2
    .byte %00011110; 3
    .byte %00001111; 4
    .byte %00000111; 5
    .byte %00000011; 6
    .byte %00000001; 7

PreDrawn4

;\--> merges with next table... don't move 'em!        

        REPEAT 16
        .byte 0
        REPEND

;\--> merges with next table... don't move 'em!        

PreDrawn2

        REPEAT 4
        .byte 0
        REPEND
        
;\--> merges with next table... don't move 'em!        

PreDrawn1

    ;     pf1 -->   pf2 <--   pf0 <--   pf1 -->   pf2 <--
    ;     ok        flip      flip xxxx ok        flip
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000001
    .byte %00000011
    .byte %00000111
    .byte %00001111
    .byte %00011110
    .byte %00111100
    .byte %01111000
    .byte %11110000
    .byte %11100000
    .byte %11000000
    .byte %10000000

        REPEAT 16 - 8
        .byte 0
        REPEND

;\--> merges with next table... don't move 'em!        

; digits shouldn't cross page boundary...

DigitNull

        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0

Digit0
        .byte %00111100
        .byte %01001110
        .byte %10000110
        .byte %10000110
        .byte %10000110
        .byte %10000110
        .byte %01001100
        .byte %00111000

Digit1
        .byte %00001100
        .byte %00001100
        .byte %00001100
        .byte %00001100
        .byte %00001100
        .byte %00101100
        .byte %00011100
        .byte %00001100

Digit2
        .byte %11111110
        .byte %11000010
        .byte %01110000
        .byte %00011100
        .byte %00000110
        .byte %00000110
        .byte %10001110
        .byte %01111000

Digit3
        .byte %01111100
        .byte %10000110
        .byte %00000110
        .byte %00000110
        .byte %00011100
        .byte %00001000
        .byte %00000100
        .byte %01111110

Digit4
        .byte %00001100
        .byte %00001100
        .byte %11111110
        .byte %10001100
        .byte %01001100
        .byte %00101100
        .byte %00011100
        .byte %00001100


Digit5
        .byte %01111100
        .byte %10000110
        .byte %00000110
        .byte %00000110
        .byte %00001100
        .byte %01111000
        .byte %01100000
        .byte %01111110

Digit6
        .byte %01111100
        .byte %11000110
        .byte %11000110
        .byte %11000110
        .byte %11111100
        .byte %01100000
        .byte %00110000
        .byte %00001100

Digit7
        .byte %00110000
        .byte %00110000
        .byte %00110000
        .byte %00110000
        .byte %00011000
        .byte %00001100
        .byte %10000110
        .byte %01111110

Digit8
        .byte %01111100
        .byte %11000110
        .byte %11000110
        .byte %11000110
        .byte %01111000
        .byte %01001100
        .byte %01001100
        .byte %00111000

Digit9

        .byte %00111000
        .byte %00001100
        .byte %00000110
        .byte %01111110
        .byte %11000110
        .byte %11000110
        .byte %01000100
        .byte %00111000

        .byte %00000000
        .byte %11101110
        .byte %11101110
        .byte %11101110
        .byte %00000000
        .byte %11101110
        .byte %11101110
        .byte %11101110


;        .byte %00000000
;        .byte %10101010                 ; level icon
;        .byte %00000000
;        .byte %10101010
;        .byte %00000000
;        .byte %10101010
;        .byte %00000000
;        .byte %10101010

        .byte %00000000
        .byte %01111100
        .byte %11000010
        .byte %10111100
        .byte %11111110
        .byte %01101010
        .byte %01111110
        .byte %00111100                 ; actual data


;        .byte %01101100         ; good player display
;        .byte %01001000
;        .byte %11111100
;        .byte %11000110
;        .byte %11100010
;        .byte %01011110
;        .byte %01110100
;        .byte %00111000


;------------------------------------------------------------------------------

OwnershipBit    .byte %00000001,%00000100,%00010000,%01000000
TargetBit       .byte %00000010,%00001000,%00100000,%10000000
TargetOwnerBit  .byte %00000011,%00001100,%00110000,%11000000

;------------------------------------------------------------------------------

TargetIndex

        ; Points to the correct line in the TargetDisplay playfield
        ; Indexed by the scanline (0-18) being drawn.

        ; This allows the single kernel to draw the entire playfield (!)

        .byte MTA_31      ; 0             blank
        .byte TargetDisplay+0      ; 1             top row target
        .byte TargetDisplay+0      ; 2             top row target
        .byte TargetDisplay+2      ; 3             top middle row target
        .byte TargetDisplay+2      ; 4             top middle row target
        .byte TargetDisplay+4      ; 5             bottom middle row target
        .byte TargetDisplay+4      ; 6             bottom middle row target
        .byte TargetDisplay+6      ; 7             bottom row target
        .byte TargetDisplay+6      ; 8             bottom row target
        .byte MTA_31      ; 9             blank
        .byte MT_TargetMask      ; 10            "
        .byte MT_TargetMask      ; 11            "
        .byte MT_TargetMask      ; 12            "
        .byte MT_TargetMask      ; 13            "
        .byte MT_TargetMask      ; 14            "
        .byte TargetDisplay+8      ; 15            cube sliding on bottom
        .byte TargetDisplay+8      ; 16            "
        .byte TargetDisplay+8      ; 17            "
        .byte TargetDisplay+8      ; 18            "

MTargetMaskInit
        .byte 0,15,15,15,15,15,15,15,15,0,0,0,0,0,0,255,255,255,255


;------------------------------------------------------------------------------

bgcol1     .byte    0 ;11000000
pfcol1    .byte    %00100100


;------------------------------------------------------------------------------

Div5

div set 0
        REPEAT 4
        REPEAT CUBEXY
        .byte div
        REPEND
div set div+1
        REPEND

Mod5

        REPEAT 4
div set 0
        REPEAT CUBEXY
        .byte div
div set div+1
        REPEND
        REPEND

;------------------------------------------------------------------------------


        ; Sprite animation is a programmed sequence
        ; Special values (*NOT* available for actual graphics)...

        ; there's colour multiplexing
        ; and there's animation...



SpriteVector

        ; indexed by creature ID/type
        ; points to the sprite shape/definition(s) for each creature

        .word HumanSprite                  ; 0
        .word Watermelon                   ; 1
;        .word Pineapple                    ; 2
;        .word Octopus                      ; 3

;------------------------------------------------------------------------------


; Sprite Graphics...

        ; SPRITE DATA MUST *NOT* CROSS PAGE BOUNDARY!!

; Format:

        ; byte          meaning
        ; 0             ALWAYS 0        used for blank display
        ; 1             vector (offset) to 2nd shape (can be 0, indicating THIS shape)
        ;               \--> allows multi-colour sprites by using 2 shapes, 2 colours
        ; add the vector if needed, and the offsets from THAT point are...
        ; 0             COLOUR of this frame
        ; ...           frame definitions
        ; ONLY for the 1st shape, 10 '0' bytes, used for rising out of cubes
        ; if the creature doesn't rise, then these bytes are NOT needed
        ; if it does, then they ARE needed for each multiplex colour/shape definition



HumanSprite

        .word SS_Normal          ; normal
        .word SS_Jump            ; jump
        .word SS_Sad             ; sad

SS_Normal

        IF PLAYER_RISES
                IF PLAYER_HAS_HAIR
                .byte 22
                ELSE
                .byte 0
                ENDIF
        ELSE
                IF PLAYER_HAS_HAIR
                .byte NO_RISE+12
                ELSE
                .byte NO_RISE+0
                ENDIF
        ENDIF
        
        .byte %00001100                 ; white

        IF PLAYER_RISES
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
                .byte %00000000
        ENDIF

testsprite
        .byte %00111100                 ; actual data
        .byte %01111110
        .byte %01110101
        .byte %11111111
        .byte %11011110
        .byte %11100001
        .byte %00111110
        .byte %01101100
        .byte %01000100
        .byte %01110110
        .byte BLANK_LINE

        IF PLAYER_HAS_HAIR
       
                .byte %00001100                 ; white

                IF PLAYER_RISES
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                        .byte %00000000
                ENDIF

                .byte %00000000                 ; actual data
                .byte %00011110
                .byte %00010101
                .byte %00111111
                .byte %11011110
                .byte %11100001
                .byte %00111110
                .byte %01101100
                .byte %01000100
                .byte %01110110
                .byte BLANK_LINE

        ENDIF


SS_Jump

        .byte NO_RISE+0                 ; no multiplex offset
        .byte %00001100                 ; white

        .byte %00000000
        .byte %00111100                 ; actual data
        .byte %01111110
        .byte %01110101
        .byte %11111111
        .byte %11011000
        .byte %11100001
        .byte %00111110
        .byte %01110011
        .byte %00000000
        .byte BLANK_LINE


SS_Sad

        .byte NO_RISE+0                 ; no multiplex colour
        .byte %00001100                 ; white

        .byte BLANK_LINE
        .byte %00111100                 ; actual data
        .byte %01111110
        .byte %01110101
        .byte %11111111
        .byte %11110000
        .byte %11101111
        .byte %00111110
        .byte %01101100
        .byte %01000100
        .byte %01110110

;------------------------------------------------------------------------------

Octopus
;Octopus - average
 
      ;  .byte %00000000

      ;  .byte %00111100
      ;  .byte %01111110
      ;  .byte %11011011
      ;  .byte %11011011
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %00111100
      ;  .byte %11101110
      ;  .byte %10101011
      ;  .byte %01001001

      ;  .byte %01100110                ; greenish

      ;  .byte %00000000
      ;  .byte %00111100
      ;  .byte %01111110
      ;  .byte %11111111
      ;  .byte %11111111
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %00111100
      ;  .byte %11111110
      ;  .byte %11111111
      ;  .byte %01101011

      ;  .byte %01100010                ; greenish

Watermelon

        .word Pineapple ;Wat0              ; normal
        .word Pineapple         ; jump
;        .word Wat0             ; happy

Wat0
        ; This is a multicolour sprite
        ; OK 17/2

        .byte 22                        ; offset to 2nd shape

        .byte %00100110                 ; red flesh

        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000

        .byte BLANK_LINE
        .byte %00000000
        .byte %00000110
        .byte %00011110
        .byte %00111110
        .byte %00111110
        .byte %00111110
        .byte %00111110
        .byte %00011110
        .byte %00000110
        .byte %00000000

        .byte %11000111                ; green rind

        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000

        .byte BLANK_LINE
        .byte %00000110
        .byte %00011000
        .byte %00100100
        .byte %01001000
        .byte %01100100
        .byte %01000000
        .byte %01010000
        .byte %00100100
        .byte %00011000
        .byte %00000110


   
Pineapple

        .byte NO_RISE+12

        IF COMPILE_VERSION = PAL
                .byte %01100111                 ; red flesh
        ELSE
                .byte %01001000                 ; red flesh
        ENDIF
;               PAL             NTSC
; 0000
; 0001          grey            light yellow
; 0010          orange          orange
; 0011          lime green      peach
; 0100          orange          pink
; 0101          blue green      purple
; 0110          pink red        dark blue
; 0111          aqua            blue
; 1000          purple          blue
; 1001          aqua            blue
; 1010          purple          aqua
; 1011          blue            green/blue
; 1100          blue purple     melon green
; 1101          dk.blue         olive
; 1110          grey            lt.green
; 1111          grey            orangy


        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00111100
        .byte %01111110
        .byte %01111110
        .byte %01111110
        .byte %01111110
        .byte %01111110
        .byte %00111100
        .byte BLANK_LINE

        IF COMPILE_VERSION = PAL
                .byte %00110111                ; green leaves
        ELSE
                .byte %11101000
        ENDIF

        .byte %01010111
        .byte %10111001
        .byte %00011010
        .byte %00111100
        .byte %01011110
        .byte %01110110
        .byte %01011010
        .byte %01101110
        .byte %01010110
        .byte %00111100
        .byte BLANK_LINE


   

;--------
; start
; program entry point





start
        sei
        cld

    ; clears all variables, stack
    ; init stack pointer
    ; also clears tia registers
    ; does this by "wrapping" the stack - unusual

    ; don't place this in a subroutine - you dunno what the stack is!

        ldx #0
        txs
        pha                     ; best way to get sp=$ff, x=0

        txa
clear   pha
        dex
        bne clear

    ; 9 bytes total for clearing stack, memory
    ; stack pointer now $ff, a=x==0


        ; FOLLOWING IMPLICITLY INITIALISES 0 BY ABOVE...

;        sta SWACNT              ; all input bits



        ;--------
        ; one-time initialization
        ; happens at system reset, no other time

        lda #$6D
        sta rnd
        lda #23
        sta rnd+1


        lda #%01101000
        sta level

        lda #0
        sta Cubes                       ; << may not be needed: CHECK

        jsr NextLevel

        lda Cubes
        ora #CUBES_DEMO_MODE
        sta Cubes

    ; main loop - this is the whole shebang

main    jsr verticalblank       ; start the display
        jsr DrawScreen
        jsr overscan
        jmp main

;--------
; verticalblank

    ; writes the vertical synch signal for the start of a frame
    ; sets up timer so we can keep track of available time - note
    ; there are 37 lines available before the screen needs to start
    ; drawing - approx 2800 cycles therefore available for game
    ; calculations

verticalblank

        lda #D1+D6
        sta VBLANK              ; start with tia disabled


    ; typical frame consists of 3 vertical sync (VSYNC) lines,
    ; 37 vertical blank (vblank) lines,
    ; 192 tv picture lines
    ; 30 overscan lines

    ; ref: nick bensema's wonderful "how to draw a playfield"

    ; require 3 VSYNC lines at start of frame
    ; ref: spg pp.1

        lda #D1
        sta WSYNC
        sta VSYNC               ; start vertical synch

    ; there's 76 cycles available on each of these lines
    ; three required for the VSYNC - don't cut it too close
    ; but do interleave other code here

        sta WSYNC               ; 1st
        sta WSYNC               ; 2nd   TV SYNCH (2 tested OK on PAL, 1 FAILED)
        sta WSYNC               ; 3rd, to satisfy ES :)


    ; we have just 37 (more) lines (@2812 cycles) of vblank, so we have to
    ; make sure the screen draw doesn't get stomped on.  the timer will tell
    ; us if we've screwed up.  timer done just before the last WSYNC to
    ; give maximum possible cycle count availability/accuracy

        IF COMPILE_VERSION = PAL
        lda #70                 ; PAL
        ELSE
        lda #30                 ; NTSC
        ENDIF

        sta TIM64T              ; 40/48 scan lines -2 (above)


        lda #0
        sta VSYNC               ; stop vertical synch


    ; screen is now scanning, but tia suppressed
    ; begins again when 0 -> vblank

    ; this is running after the TV VSYNC has been sent, but before
    ; we need to start drawing lines.  there are 37 lines, just around
    ; 2800 cycles.  gotta be sure sure that we don't overflow that time
    ; otherwise we'll get screen jitter

        lda BackColour
        sta COLUBK

        lda level
        asl
        asl
        asl
        asl
        ora #%0100

        IF ATTRACT_MODE
                eor BackColour          ; gives demo effect
        ENDIF
        
        sta COLUPF

                ; << lots of time in here to do stuff
                ; (approx 2800 cycles)

        ldx #31
        ldy #15
        jsr FeedCube
        bmi endvb

        lda #%00111111
        sta CubeYDir,x                  ; moving left, bottom of screen


endvb   lda INTIM
        bne endvb

        sta WSYNC
        sta VBLANK              ; end vblank period
;        sta WSYNC

        rts

;------------------------------------------------------------------------------


DrawScreen

    ; the kernel
    ; line by line we have to hold hands with the tia, passing it data
    ; so that it draws the screen for us.
    
    ; wait for vertical blank period to end
    ; timer counts down 1 every 64 cycles, so just wait for it
    ; it may already have expired


        lda #%11111100
        sta COLUP0
        sta COLUP1

;        lda #233
;        sta TIM64T                     ; 200.4 scan lines

        sta WSYNC                       ; not 100% required


        IF PROTECT_SCORELINE

                ; Don't show the score if player sprites would overlay
                ; will overlay if the player is jumping on the top row, or from the top row

        ldx #1
OverSp  lda PlayerCounter,x
        and #%11100000
        cmp #MODE_JUMP                  ; jumping mode?
        bne NoJ
        lda PlayerCube,x
        and #7
        tay

        lda CubeYDir,y
        and #15
        beq NoScore3
        cmp #6
        bcs NoJ

                ; Cater for on 2nd row of cubes, jumping upward

        lda PlayerFlags,x
        and #%11100000
        cmp #%01000000
        beq NoScore3 

NoJ     dex
        bpl OverSp

ShowScore

        ENDIF


        ;**************************************************
        ; WARNING: VARIABLE USAGE
        ; DO *NOT* PLACE THIS BETWEEN OVERLAY VAR USAGE

        jsr SetupTopScore
        sta WSYNC

        lda #%00001111
        jsr ScoreLine           ; <<< OVERWRITES *ALL* OVERLAY VARS

        ;**************************************************

        jsr SpriteHandling

        IF PROTECT_SCORELINE

        lda #0
        sta MTA_ZERO
        sta MTA_ZERO+1

        lda #31
        sta MTA_31
        sta MTA_31+1

        ldy #20                 ; size of score line
        jmp NoScore2



NoScore3

        sta WSYNC
        sta WSYNC

      

        jsr SpriteHandling
        
LINESUSED       = 13

        lda #31
        sta MTA_31
        sta MTA_31+1

        ldy #0
        sty MTA_ZERO
        sty MTA_ZERO+1


        ldx #0
        REPEAT LINESUSED
        jsr MBlock
        REPEND

NoScore2

        ENDIF


        ldy #LINESUSED

;------------------------------------------------------------------------------


    ;----
    ; cycle thru drawing line by line...
    ; time is precious here... there are just 228/3 cycles per line
    ; (giving us just 76 cycles) and that's not counting the WSYNC,
    ; should you be writing that. There are 68 pixels (/3 = cycles)
    ; in the hblank period, which starts at the conceptual beginning
    ; of a line on this machine

        lda #0 ;31 ;15
        sta MT_TargetMask       ; mask for border of target area

        ldx #3                  ; border
        jsr MBlock
        jsr MBlock
        jsr MBlock
        jsr MBlock

        
        ; Draw the screen with the associated target area display        
        ; Timing is INCREDIBLY TIGHT between these routines.


                ; INNER TARGET SECTION - 8 "LINES"

        ldx #0                          ; << character line



        jsr MTargetArea                 ; border

        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea                 ; drawing a line (with target area)

        jsr MTargetArea                 ; border

        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea         ; drawing a line (with - effectively NO - target )

        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea
        jsr MTargetArea         ; drawing the bottom 4 lines with the 'slide-on' cube

        ldx #3                  ; Bottom border
        jsr MBlock
        jsr MBlock
        jsr MBlock
        jsr MBlock

        lda #0
        sta PF0


        ;**************************************************
        ; WARNING: VARIABLE USAGE
        ; DO *NOT* PLACE THIS BETWEEN OVERLAY VAR USAGE

        jsr SetupBottomScore
        lda #%00101011
        jsr ScoreLine           ; <<< OVERWRITES *ALL* OVERLAY VARS

        ;**************************************************

    ; make sure we have a 192 line display
    ; pal must have an even # or the colours screw up?!

;EOScr   lda INTIM
;        bne EOScr

        sta WSYNC

        lda #D6+D1
        sta VBLANK              ; end of screen - enter blanking
    
        rts

;------------------------------------------------------------------------------

overscan

        sta WSYNC

        IF COMPILE_VERSION = PAL
        lda #61                 ; 30+ scan lines of overscan
        ELSE
        lda #42                 ; NTSC
        ENDIF
        
        sta TIM64T
        
        dec timer               ; one tick
        bne nHTimer
        ldx timer+1
        beq nHTimer
        dec timer+1
nHTimer

        lda timer
        and #1
        bne MCLimit             ; limit speed of *ALL* cube movement

        jsr MoveCube
        cmp ThankES

MCLimit

        jsr CreatureAI

ThankES lda INTIM
        bne ThankES

        sta WSYNC
        rts

;------------------------------------------------------------------------------


SpriteHandling

        ; Sets up the player sprites - called at the top of screen just before sprites
        ; If there's a score on display, its after the score


        ; Setup vars for graphics draw of sprites
        ; HORRIBLY INEFFICIENT-LOOKING

        ; setup (sprite+x) to account for new draw system
        ; must point n bytes BEFORE definition, and still not cross page boundary (sheesh)
        ; also setup SpriteEnd pointers appropriately


        ldx #1

SetupSprite

                ; Mirror the sprite IFF the object is/has been facing left
                ; The mirroring is currently cleared for digit display in ScoreLine

        lda PlayerFlags,x
        and #FLAG_MIRROR
        lsr
        sta REFP0,x                     ; set mirror appropriately

        lda PlayerCube,x
        lsr
        lsr
        and #%1110                      ; ID/type
        tay

        lda SpriteVector,y
        sta DS_Sprite
        lda SpriteVector+1,y
        sta DS_Sprite+1                 ; --> vector to p2

        lda PlayerFlags,x
        and #%1110
        tay
        lda (DS_Sprite),y               ;--> table of animation pointers
        pha
        iny
        lda (DS_Sprite),y
        sta DS_Sprite+1
        pla
        sta DS_Sprite                   ; --> frame of animation

                ; Frame block data consists of
                ; +0                    D6-D0 byte offset to multiplex pattern - 1
                ;                             points to the start of the sprite frame (colour)
                ;                       D7    this frame has "rising" capability

                ; --> single frame itself consists of...
                ; +0                    colour of frame
                ; +1-9                  (OPTIONAL, see D7 above) 0-bytes allow "rising" capability
                ; +10 (or 1)            BLANK_LINE (ie: 0) required by sprite system
                ; +11-20 (or 2-11)      10 lines of sprite data


        lda timer
        lsr                             ; D0 --> C

        ldy #0
        lda (DS_Sprite),y
        sta DS_temp                     ; capture rising-capable (D7) + offset

        bcc NoMulti
        and #$7f
        tay                             ; offset to multiplex pattern #2
NoMulti

        iny                             ; --> colour
        lda (DS_Sprite),y
        sta COLUP0,x                      

                ; Now adjust for the actual 'rising' value IFF the frame is capable

        lda DS_temp                     ; D7 = rising-capable?
        bmi NotRisingx7
 jmp NotRisingx7 ;tmp

        tya
        clc
        adc #SPRITE_HEIGHT
        tay                             ; --> BLANK_LINE

        lda PlayerFlags,x
        and #FLAG_RISE
        beq NotRisingx7


                ; The player WANTS to use rising, and the frame CAN use rising...
                ; so we shift back our sprite index by the amount we want to SINK by

        lda PlayerCounter,x
        and #15
        sta DS_temp                     ; rise amount

        tya
        sec
        sbc DS_temp
        tay

NotRisingx7

        sty DS_temp                     ; --> start of the 10 bytes of frame data

        txa
        asl
        tay

        sec                             ; yes, +1
        lda DS_temp
        adc DS_Sprite
        sta Sprite,y
        lda DS_Sprite+1
        adc #0
        sta Sprite+1,y

                ; Have finished calculating what shape to display, now WHERE (vertically) to display it?

        lda PlayerY,x
        cmp #128
        ror
        sta SpriteEnd,x                ; just a temp (=HalfY)

        lda PlayerCube,x
        and #7
        tay                             ; just cube index

        lda CubeYDir,y
        and #15                         ; cube Y coord
        sta DS_temp
        asl
        adc DS_temp                     ; 3y (3 sprite lines/1 y for cubes)
        sec
        sbc SpriteEnd,x

        adc #SPRITE_Y_ADJUST
        sta SpriteEnd,x                 ; double-scanline of end of sprite
     
        txa
        asl
        tay

        sec
        lda Sprite,y
        sbc SpriteEnd,x
        sta Sprite,y
        lda Sprite+1,y
        sbc #0
        sta Sprite+1,y

        clc
        lda Sprite,y
        adc #SPRITE_HEIGHT
        sta Sprite,y
        lda Sprite+1,y
        adc #0
        sta Sprite+1,y

        dex
        bmi DoneC
        jmp SetupSprite


DoneC


PositionSprites

        ; Set the horizontal position of the two sprites, based upon their coordinates
        ; This uses the tricky 2600 method of positioning (RESPx, HMPx, etc)

        ; Algorithm invented a looong time ago by persons unknown
        ; Re-invented Feb2001 by yours truly, then optimised according to code by Thomas Jenztsch

        sta WSYNC
        sta HMCLR                       ; clear any previous movement

        ldx #1                          ; sprite index
PosSP   
        lda PlayerX,x
        tay

        lsr
        lsr
        lsr
        lsr
        
        sta PS_temp

        tya
        and #15

        clc
        adc PS_temp

        ldy PS_temp

        cmp #15
        bcc NH
        sbc #15
        iny
NH

                ; Use remainder for fine adjustment
        eor #7
        asl
        asl
        asl
        asl

        sta HMP0,x                      ; fine movement
        sta WSYNC

        jsr Ret
        bit 0                           ; 15 cycles = 3 loops :)

Jiggle  dey
        bpl Jiggle

        sta RESP0,x

        dex
        bpl PosSP

        sta WSYNC
        sta HMOVE
        sta WSYNC

        lda #0 ;D0
        sta VDELP0

        lda #0
        sta CTRLPF
        sta NUSIZ0
        sta NUSIZ1

        rts



;------------------------------------------------------------------------------

Owned   ldx CM_CubeIndex
        ldy CM_Direction

        sec                     ; fail
        rts


CheckMove

        ; Determine if it is possible to move in a desired direction
        ; if so, current position is vacated and new position claimed


        ; y = direction (guaranteed 0-3, preserved)
        IF SLOW_AUTOMOVE
                ;       bit D2 = 1 means slow movement
        ENDIF

        ; x = cube index (preserve!)  MUST BE A LEGIT CUBE, NOT JUST AN INDEX
        ; AI_Player = player# (could be FF)


        ; returns carry C = success
        ; S = fail
        ; X,Y preserved

        stx CM_CubeIndex
        sty CM_Direction
                
        IF SLOW_AUTOMOVE
        
                tya
                and #3
                tay                     ; drop speed bit D2
        
        ENDIF
        
         ; apply movement

        lda CubeX,x
        and #31
        cmp #16
        bcs Owned               ; offscreen - no moving
        tax

        lda DirMoveX,y
        adc Div5,x              ; now a horizontal index (0-3)  (RELY ON CARRY C)
        cmp #4                  ; an illegal position?
        bcs Owned               ; can't move there

        sta CM_Column

        ldx CM_CubeIndex
        lda CubeYDir,x
        and #15
        tax

        lda DirMoveY,y
        adc Div5,x              ; (RELY ON CARRY C)
        cmp #4                  ; an illegal position?
        bcs Owned               ; can't move there
        
        tay
        ldx CM_Column


        lda Grid,y              ; row of grid
        and OwnershipBit,x
        bne Owned               ; can't go there!


                ; OK, so the grid is available... grab it!

        lda Grid,y
        ora OwnershipBit,x
        sta Grid,y

        ldy #SOUND_SLIDING
        jsr Sound               ; sliding


                ; Score for player moving cube to correct position

        and TargetBit,x
        beq NoGoal

        lda PlayerCube
        eor CM_CubeIndex
        and #7
        bne NoGoal                      ; not player's cube

        ldy #0
        lda #$10
        jsr AddScore

NoGoal

                ; free the grid currently occupied

        ldx CM_CubeIndex

        lda CubeYDir,x
        and #15
        tay
        lda Div5,y
        tay

        lda CubeX,x
        and #31                 ; GUARANTEED <=15
        tax
        lda Div5,x
        tax

        lda OwnershipBit,x
        eor #$FF
        and Grid,y
        sta Grid,y
OffSc

                ; Merge new direction into cube information

        lda CM_Direction
        tay
        IF SLOW_AUTOMOVE
                and #3
        ENDIF

        asl
        asl
        asl
        asl
        sta CM_scratch

        ldx CM_CubeIndex
        lda CubeYDir,x
        and #%00001111                  ; keep all but direction bits
        ora CM_scratch
        sta CubeYDir,x

        IF SLOW_AUTOMOVE

                lda CubeX,x
                and #%10111111
                ldy CM_Direction
                cpy #4
                bcc RmSlow
                ora #%01000000
RmSlow          sta CubeX,x

        ENDIF

        clc
        rts


;------------------------------------------------------------------------------


CheckJump
; Optimized 14/2

        ; pass X = cube index
        ; Y = direction (only use 0-3 - out of range NOT checked)
        ; AI_Player = index of player doing the jump
        ; Tries to jump creature in desired direction
        ; returns: x,y buggered

        sty CJ_Direction

        lda CubeX,x
        bmi NoJump              ; dead cube (!)
        lda CubeYDir,x
        bpl NoJump              ; my cube is moving
        and #15

        clc
        adc DirMove5Y,y         ;  now a vertical coordinate
        cmp #16                 ; an illegal position?
        bcs NoJump              ; can't move there

        sta CJ_NewY

        lda CubeX,x
        and #31
;        cmp #16
;        bcs NoJump              ; feeding - not a valid source (already catered, above)

        adc DirMove5X,y         ; CC assumed... now a horizontal coordinate
        cmp #16                 ; an illegal position?
        bcs NoJump              ; can't move there

        sta CJ_NewX


                ; Find the cube occupying that grid position (if any)

        ldx #NUMBEROFCUBES-1

FindTarget

        lda CubeX,x
        bmi NextTarget          ; dead cube
        
        lda CubeYDir,x
        bpl NextTarget          ; potential destination is moving - fail
        and #15
        cmp CJ_NewY
        bne NextTarget          ; fails Y comparison

        lda CubeX,x
        and #31
        cmp CJ_NewX
        beq GotTarget           ; succeeds on X,Y

NextTarget

        dex
        bpl FindTarget
        rts

GotTarget
              
        stx CJ_CubeNumber
 
        ldy AI_Player
        bmi NoJump                      ; virtual

        lda PlayerCube,y
        and #%11111000
        ora CJ_CubeNumber
        sta PlayerCube,y                ; record new cube for player

        lda #MODE_JUMP + 10
        sta PlayerCounter,y

        lda PlayerFlags,y
        and #%00010001
        sta PlayerFlags,y

        ldx CJ_Direction
        lda AdjustY,x
        sta PlayerY,y

                ; store the jump direction
        
        txa
        lsr
        ror
        ror
        ror
        and #%11100000
        ora PlayerFlags,y
        ora #%00000010                  ; jump frame = #1
        sta PlayerFlags,y               ; set direction of jump

NoJump  rts


AdjustY .byte -15,0,25,0,0,0,0,0        ; <<< FIX DIAGONALS

;------------------------------------------------------------------------------

SetOwner

        ; Markes a grid position as occupied.  Prevents other cubes moving there.
        ; pass X = valid cube index (junked)

        lda CubeYDir,x
        and #15
        tay
        lda Div5,y
        tay

        lda CubeX,x
        and #31
        cmp #16
        bcc XinRange
        lda #15                         ; far-right (ie: feeding-on) cubes are trimmed to ON
XinRange
        tax
        lda Div5,x
        tax

        lda OwnershipBit,x
        ora Grid,y
        sta Grid,y
        rts

;------------------------------------------------------------------------------

Random
        ; Generate a pseudo-random number

       lda rnd
       lsr
       lsr
       sbc rnd
       lsr
       ror rnd+1
       ror rnd
       ror rnd
       lda rnd
       eor timer
       rts


;------------------------------------------------------------------------------

TieToCube

        ; Horizontally position a sprite to where a cube is
        ; Screwy sprite positioning by the hardware makes this a bit tricky

        ; X = sprite object to tie (0 or 1) (preserved)


        lda PlayerCube,x
        and #7
        tay

        lda CubeX,y
        and #31
        asl
        asl
        adc #SPRITE_X_ADJUST
        sta PlayerX,x

        rts



TargetDisplayBitL

        .byte %00001100
        .byte %00000011

;\--> merges with next table... don't move 'em!        

TargetDisplayBitR

        .byte %00000000
        .byte %00000000
        .byte %00000011
        .byte %00001100


;------------------------------------------------------------------------------

CheckTableaux

        ; Change the TargetDisplay so that cubes flash appropriately
        ; CHECKS/UPDATES ONE SQUARE PER CALL

                ; This next bit is using varied bits of the counter to determine which
                ; of the cubes in the target display is to be checked.  By mangling the
                ; counter, we get the effect not only of a regular (slowish) flash rate,
                ; but we also stagger the order in which things are updated - and thus
                ; get a small but discernable difference between the cubes updates.
                ; This small effect gives some visible border between adjacent cubes
                ; but still allows the entire to flash seemingly simultaneously
    

                ; When time runs out, generate new tableaux
                ; This is also the first-time init of the tableaux

        lda Cubes
        and #CUBES_COMPLETED
        ora timer+1
        bne noResetD
        jsr ResetTableaux               ; generate new pattern, init timer

noResetD

                ; Flash the border of the target area if we're running out of time
                ; (OR if screen complete) Pretty neat optimisation

        lda Cubes
        and #CUBES_COMPLETED
        cmp #CUBES_COMPLETED
        beq FT2a

        lda #1
        cmp timer+1
FT2a    lda #8
        tay
        bcc FTargA                      ; not time... normal display instead
        and timer
FTargA  asl
DClear  sta TargetDisplay-1,y
        dey
        bne DClear

                ; How about that, huh!



        lda #15
        sta CT_index

CTIter
        lda CT_index
        and #3
        tax
        lda CT_index
        lsr
        lsr
        tay

        lda Grid,x
        and TargetBit,y
        beq Flasher             ; no target there

        lda Grid,x
        and TargetOwnerBit,y
        cmp TargetOwnerBit,y
        beq Correct             ; correct square, correct position

        ; So there's a target, but no cube.  It has to flash

        lda timer
        and #16
        bne Flasher             ; toggle controls flash rate

Correct

        txa
        asl
        tax

        ; Turn on any display for this square

        lda TargetDisplayBitL,y
        ora TargetDisplay,x
        sta TargetDisplay,x

        lda TargetDisplayBitR,y
        ora TargetDisplay+1,x
        sta TargetDisplay+1,x

Flasher

        dec CT_index
        bpl CTIter




        rts

;UpdateX .byte 0,0,1,1,2,2,3,3,0,0,1,1,2,2,3,3
;UpdateY .byte 0,2,1,3,0,2,1,3,1,3,0,2,1,3,0,2


;------------------------------------------------------------------------------


MBlock

        ; preserve y (+1)

     ; LINE 1 -------------------------------

        sta WSYNC               ; 3

        lda #0                  ; 2
        sta PF0                 ; 3 ->8              (@5)
        sta PF1                 ; 3 ->11        < 28 (@8)
        sta PF2                 ; 3 ->14        < 39 (@11)

        jsr Ret                 ; 12
        jsr Ret                 ; 12
        jsr Ret                 ; 12
        bit 3
        bit 3
        nop
;        nop

        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd           ; 3
        adc #SPRITE_HEIGHT      ; 2
        bcs .MBDraw3              ; 2(3)

        nop                     ; 2
        nop                     ; 2
        sec                     ; 2
        bcs .skipMBDraw3          ; 3

.MBDraw3
        lda (Sprite),y          ; 5
        sta PLAYER0SHAPE        ; 3

.skipMBDraw3

        ;-->18 cycles used
        ;--------------------------

;->76

     ; LINE 2 -------------------------------

        sta WSYNC               ; 3

        lda BlockIndex,x        ; 4
        sta PF0                 ; 3
        lda BlockIndex+1,x      ; 4
        sta PF1                 ; 3 ->17        < 28 (@14)
        sta PF2                 ; 3 ->20        < 39 (@17)



    ; right pf




        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd+1         ; 3
        adc #SPRITE_HEIGHT      ; 2

        lda BlockIndex+2,x      ; 4
        sta PF0                 ; 3 ->41        < 49 (@37)

        bcs .MBDraw2            ; 2(3)

        nop                     ; 2
        nop                     ; 2
        sec                     ; 2
        bcs .skipMBDraw2        ; 3

.MBDraw2
        lda (Sprite+2),y        ; 5
        sta PLAYER1SHAPE        ; 3

.skipMBDraw2

        lda #0                  ; 2
        sta PF1                 ; 3
        sta PF2                 ; 3 ->54[5]     < 66 (@49[50])


        ;-->18 cycles used
        ;--------------------------


        iny                     ; 2                             NEXT LINE FOR SPRITES
        rts



BlockIndex

        ; Determines WHAT is to be displayed on the lines

        .byte %00000000,%00000000,%00000000
        .byte %10000000,%11111111,%11110000


;------------------------------------------------------------------------------



FeedCube

        ; Feed a new cube onto the grid
        ; Count the number of cubes actually IN ACTION.
        ; If there's more to come on, AND the bottom right square is free...
        ; FEED on a cube

        ; pass Y = row (0-15), X = column (0-15) or 31 for special feeder

        stx FC_CubeX
        sty FC_CubeY

        ; returns X = cube index of fed cube (or $FF if none)

        ldx #$FF
        stx FC_FedCubeIndex             ; returned index



        ldy #0                          ; counts cubes-in-action
Feeding inx

        lda CubeX,x
        bpl Count                       ; alive?

        stx FC_BlankSpot                ; a blank spot
        dey                             ; NOT counted

Count   iny                             ; count cubes
        cpx #NUMBEROFCUBES-1
        bcc Feeding

        ; Y now holds count of cubes which are active
        ; 'FC_BlankSpot' will be $FF if all cubes are active (no index found!)

        tya
        eor Cubes
        and #7
        beq NotVacant                   ; already all active!

                ; There's still cubes to come... possibly show a pending cube at right edge
                ; See if the previous cube is fully fed, first...

        ldx #NUMBEROFCUBES-1
FSlider lda CubeX,x
        bmi nValid                      ; dead cube
        and #31
        cmp #24
        bcs NoPend
nValid  dex
        bpl FSlider

        lda #%11110000
        sta TargetDisplay+9             ; show the static 'pending' cube

NoPend

        lda FC_CubeX
        cmp #16
        bcc NotSlider
        lda #15
NotSlider
        lsr
        lsr
        tax

        lda FC_CubeY
        lsr
        lsr
        tay

        lda Grid,y
        and OwnershipBit,x
        bne NotVacant                   ; can't feed: bottom-right square occupied!


        ; Start cube bottom-right, feeding in to screen

        ldx FC_BlankSpot                ; last blank spot

        lda FC_CubeX
        sta CubeX,x
 
        lda FC_CubeY
        ora #$80                        ; stationary
;       lda #%00111111
        sta CubeYDir,x                  ; moving left, bottom of screen

        ldy #SOUND_SLIDING
        jsr Sound

        jsr SetOwner
        ldx FC_BlankSpot                ; last blank spot

        rts

NotVacant

        ldx FC_FedCubeIndex
        rts



;------------------------------------------------------------------------------


InitCubes

        ; Called on exit from demo mode, so also inits colours

        lda #0
        sta BackColour
        sta timer+1                     ; FORCES tableaux reset in CheckCompletion
        
        ldx #3
ClGrid  sta Grid,x
        dex
        bpl ClGrid

        sta TargetDisplay+8
        sta TargetDisplay+9                                ; SLIDING TARGET AREA

;        jsr ResetTableaux              ; now done per the normal reset (doubles as init)

        lda #%0000000
        sta PlayerCube                  ; creature ID #0
        lda #%0001000
        sta PlayerCube+1                ; creature ID #1
 

        ldx #NUMBEROFCUBES-1
        lda #D7
shiftc  sta CubeX,x                     ; dead cube
        dex
        bpl shiftc

        ldx #31
        ldy #15
        jsr FeedCube
        lda #%00111111
        sta CubeYDir,x                  ; moving left, bottom of screen


        stx IC_FedCubeIndex

        ldx #1
IPC     lda PlayerCube,x
        and #%11111000
        ora IC_FedCubeIndex
        sta PlayerCube,x

        jsr TieToCube

        dex
        bpl IPC
        rts

;------------------------------------------------------------------------------

ResetTableaux

                ; Calculated a graduated time for completing the screen

        sec
        lda #31
        sbc level
        and #%11111
        lsr
        lsr
        cmp #3
        bcs Ntfast
        lda #3
Ntfast  sta timer+1

                ; Generic inits...
        ldx #7
        lda #16
ClearT  sta TargetDisplay,x
        dex
        bpl ClearT

        lda Cubes
        and #7

        sta CT_Cubes

        ldx #3
ClGrid2 lda Grid,x
        and #%01010101                                        ; clear all (previous) home positions
        sta Grid,x
        dex
        bpl ClGrid2

CTab
        jsr RandomXYPosition
        
        lda Grid,x
        ora TargetBit,y
        sta Grid,x

        dec CT_Cubes
        bne CTab

        lda Cubes
        and #7
        lsr
        sta CT_Cubes

        rts


RandomXYPosition

                ; Select a random placement for a cube
                ; returns X,Y position
                ; DON'T MAKE THIS RANDOM/RANDOM, as it will delay/jiggle the screen

        jsr Random
        and #3
        tax
        jsr Random
        and #3
        tay

CheckT  lda Grid,x
        and TargetBit,y
        beq FTab

        dex
        bpl CheckT
        ldx #3
        dey
        bpl CheckT
        ldy #3
        bne CheckT

FTab    rts


;------------------------------------------------------------------------------

BeginGame

                ; Game-inits go here...

        lda #%01100000
        sta level                       ; level = 0, 3 lives

        lda #0
        ldx #2
ClScore sta PlayerScore,x               ; score = 0
        dex
        bpl ClScore


NextLevel



                ; Level-inits go here...

        lda level
        and #%11111

        clc
        adc #1
        cmp #30
        beq Unchange            ; maxed out

        sta NL_temp

        lda level
        and #%11100000
        ora NL_temp
        sta level

Unchange

        lda Cubes
        and #%00111000                  ; turn OFF # cubes, demo, completed flashing
        sta Cubes

        lda level
        and #%11110
        lsr
        adc #3                          ; CC!
        cmp #NUMBEROFCUBES
        bcc LimitC
        lda #NUMBEROFCUBES
LimitC

        ora Cubes
        sta Cubes

        jsr InitCubes

        ldx #0
        jsr InitCreature
        ldx #1
        jsr InitCreature

                ; Dodgy use of FC_ vars... created for this usage
                ; Prevents overlay clumsiness, but beware - this routine must be last

        lda Cubes
        and #7
        lsr
        sta FC_temp2

Plonk
                ; Plonk down some random cubes

        jsr Random
        and #3
        sta FC_coord
        asl
        asl
        adc FC_coord
        tax

        jsr Random
        and #3
        sta FC_coord
        asl
        asl
        adc FC_coord
        tay

        jsr FeedCube            ; COULD fail (over another), but we don't care


        dec FC_temp2
        bne Plonk

        rts

;------------------------------------------------------------------------------

; Place table before code (for DASM, avoiding "Label Mismatch" warnings)

joystickdir
        .byte $FF,$FF,$FF,$FF,$FF,$FF,$FF,1,$FF,$FF,$FF,3,$FF,2,0,$FF,

;------------------------------------------------------------------------------

InitCreature

        ; Start a creature.  It will slide up out of the cube
        ; pass X = creature index (0 or 1)

        lda #MODE_RISE + SPRITE_HEIGHT
        sta PlayerCounter,x

        lda #FLAG_RISE + FLAG_MIRROR
        sta PlayerFlags,x

        lda #0
        sta PlayerY,x                   ; no vertical offset

        rts

;------------------------------------------------------------------------------

CreatureAI

        ; Perform AI/logic for creatures
 
        ldx #0
        jsr DoAI

        jsr CheckTableaux       ; animate target grid

        lda Cubes
        and #CUBES_COMPLETED
        bne Finished


        ldx #1
        jsr DoAI


        lda PlayerFlags+1
        and #FLAG_RISE
        bne ranpine

        lda timer
        and #64
        bne ranpine

        lda PlayerFlags+1
        eor #%00000010
        sta PlayerFlags+1
ranpine
        



        ; And now the general AI manager....
        ; The job is to generally hamper the player

        ; Randomly shift cubes...        
        
        jsr Random
        cmp #240
        bcc NotNasty
        lda rnd+1
        and #7
        tax
        cpx #NUMBEROFCUBES
        bcs NotNasty            ; invalid index

        eor PlayerCube
        and #7
        beq NotNasty            ; don't move player's

        lda CubeX,x
        bmi NotNasty            ; dead cube
        lda CubeYDir,x
        bpl NotNasty            ; moving!

        lda #$FF
        sta AI_Player

        jsr Random
        and #3
        
        IF SLOW_AUTOMOVE
                ora #MOVE_SLOW_ENABLE   ; slow mover
        ENDIF
        
        jsr CheckMove


NotNasty
Finished

        rts


DoAI

        ; x = creature index

        stx AI_Player

        lda PlayerCounter,x

        lsr
        lsr
        lsr
        lsr

        and #%1110
        tay

        lda AIVector,y
        sta AI_Vector
        lda AIVector+1,y
        sta AI_Vector+1
        jmp (AI_Vector)

AIVector

        .word AINormal
        .word AIRiseUp
        .word AISink
        .word AILurk
        .word AIJump
        .word AILock
        .word AIDelay
        ; etc..


;------------------------------------------------------------------------------

AIJump

        ldy #SOUND_JUMP
        jsr Sound


        ; Creature is mid-jump
        ; The counter gives us the index into the jump table
        ; The entry at the table gives us the height (y offset) for this frame

;        lda PlayerCounter,x
;        and #%11111
;        cmp #3
;        bne notLand

;notLand



        lda #%0
        jsr CountDown                   ; -->y
        bcc EndJump

        tya
        and #3
        sta AIJ_YSlower

;        sec                             ; CS!
        lda PlayerY,x
        sbc JumpVector,y
        sta PlayerY,x

        lda PlayerFlags,x
        asl
        rol
        rol
        rol
        and #3
        tay                             ; direction movement

        lda DirMoveX,y
        asl
        clc
        adc PlayerX,x
        sta PlayerX,x
         
        lda AIJ_YSlower
        beq nY
            
        sec
        lda PlayerY,x
        sbc DirMoveY,y
        sec
        sbc DirMoveY,y
        sta PlayerY,x

nY      rts

EndJump

        lda #MODE_NORMAL ;DELAY + TIME_PAUSE_AFTER_JUMP
        sta PlayerCounter,x

        lda #0
        sta PlayerY,x

        lda PlayerFlags,x
        and #%11110001          ; animation #0
        sta PlayerFlags,x

        ldy #SOUND_OFF
        jsr Sound

        rts


JumpVector      .byte 10,8,6,4,2,-2,-4,-6,-8,-10

;------------------------------------------------------------------------------

Sound
        ; Sounds are mini-programs
        ; there are just two sound channels, each has a program which it runs
        ; the program consists of a volume, frequency, control, duration
        ; The duration counts number of frames
        ; Coded values allow jumping
        ; 128 bytes *MAX* for *ALL* the sounds (there's no more room anyway :)



        ; y = sound #

        lda SoundVolume,y
        sta AUDV0
        lda SoundControl,y
        sta AUDC0
        lda SoundFrequency,y
        sta AUDF0
        rts

SOUND_OFF               = 0
SOUND_SLIDING           = 1
SOUND_JUMP              = 2
SOUND_

; interesting ones...
; 4/20/2  kind of a landing






;SOUND_COMMAND_END       = 0
;SOUND_COMMAND_BRANCH    = 255

        ; Format:  D7+[delay] | SOUND_COMMAND_BRANCH, branch
        ;       D7=0+[freq], control+volume


;        ldx #1

;TwoChanSound

;       ldy SoundPP,x
;        beq nextChannel                         ; channel inactive

;ProcessSound2

;        lda SoundProgram,y
;        bpl CFV

                ; a command

;        cmp #SOUND_COMMAND_END
;        bne notEnd
;        sta SoundPP,x                           ; halts
;        bcs nextChannel

;notEnd  cmp #SOUND_COMMAND_BRANCH
;        bne notBranch
        
;        lda SoundProgram+1,y
;        tay
;        bcs ProcessSound2

;notBranch               ; assume delay, encoded

;        and #127
;        sta SoundDelay,x
;        iny
;        bne ProcessSound2
     
;CFV
;        sta AUDF0,x
;        iny
;        lda SoundProgram,y
;        sta AUDV0,x
;        lsr
;        lsr
;        lsr
;        lsr
;        sta AUDC0,x
;        iny

;        sty SoundPP,x

;nextChannel

;        dex
;        bpl TwoChanSound

;        rts


;Sound
;        .byte 0                 ; << halted


;        .byte D7+127,0,0,SOUND_COMMAND_BRANCH,0
;        .byte 





SoundVolume             ;         ID              CONTROL VOLUME
        .byte %00000000        ; 0      QUIET
        .byte %10000001        ; 1      SLIDING          8       1
        .byte %00110010        ; 2      JUMP

SoundFrequency
        .byte 0
        .byte 31                ; 1
        .byte 10                 ; 2

SoundControl
        .byte 0
        .byte 8                 ; 1
        .byte 3                 ; 2
 
;------------------------------------------------------------------------------

CountDown

        ; Generic decrement of the player's counter
        ; pass x = player, a = timer mask
        ; returns carry S = counter still active (non-zero on entry)
        ; y = new value of counter
        ; C = counter halted (was 0)

        sec
        and timer
        bne NotEnd

        clc
        lda PlayerCounter,x
        and #%11111
        beq NotEnd                      ; actually, finished (0-->CC)

        dec PlayerCounter,x

        sec

NotEnd  lda PlayerCounter,x
        and #%11111
        tay
        rts

;------------------------------------------------------------------------------

AILock

        lda #CUBES_COMPLETED
        ora Cubes
        sta Cubes

                ; Flash between jump/normal frame

        lda PlayerFlags
        and #%11110001
        sta PlayerFlags

        lda timer
        and #8
        lsr
        lsr
        ora PlayerFlags
        sta PlayerFlags

        ; Locked, pending end of screen

        lda #%111
        jsr CountDown

        lda timer+1
        beq noBonus
        dec timer+1

                ; Can't finish until bonus has been added

        ldy #$01
        lda #$00
        jsr AddScore

        sec

noBonus bcc ToEnd
        
        rts


ToEnd
        jmp NextLevel           ; paired JMP/RTS saving
        ;rts

;------------------------------------------------------------------------------


AINormal

        ; Currently just sits around waiting to sink

        cpx #0
        beq Human

;        jsr Random
;        cmp #3
;        bcs Normaling
        jsr Random
        cmp #20
;        bcs Normaling


        lda #MODE_SINK + 0
        sta PlayerCounter,x

        lda PlayerFlags,x
        ora #FLAG_RISE
        sta PlayerFlags,x               ; use counter as rise/fall value

Normaling

        rts



;------------------------------------------------------------------------------

Joystick
 
;        lda #0
;        sta SWACNT              ; all input bits (IMPLICITLY SET @ STARTUP)

        lda SWCHA               ; joystick
        lsr
        lsr
        lsr
        lsr
        tay

        lda joystickdir,y
        rts


Human

        ldy #0
        lda #$1
        jsr AddScore


        jsr Joystick
        tay

        lda Cubes
        bpl HumanMove                   ; not in demo mode?

        IF ATTRACT_MODE
                        ; attract mode just randomises background colour

                lda timer
                bne NoAttract
                lda rnd
                and #%11110111          ; not too high-intensity
                sta BackColour
NoAttract
        ENDIF

        cpy #4
        bcs NoDemo                      ; human interaction?
        jmp BeginGame                   ; fall-through jump
;        rts

NoDemo
                ; DEMO mode in control - choose a move

        lda rnd ;jsr Random
        and #31                         ; only 0..3 legal
        tay

HumanMove


                ; Check cube-in-motion - can't do anything if so

        lda PlayerCube,x
        and #7
        tax
        lda CubeYDir,x
        bpl UnJoy                       ; Can't do anything if cube in motion

        
        cpy #4
        bcs UnJoy                       ; no legal joystick direction

                ; Handle facing Left/Right

        cpy #1
        bne NoMirC
        lda PlayerFlags
        and #%11101111
        sta PlayerFlags                 ; turn OFF mirroring
NoMirC

        cpy #3
        bne NoSetC
        lda PlayerFlags
        ora #%00010000
        sta PlayerFlags                 ; turn ON mirroring
NoSetC

        tya
        jsr CheckMove        
        bcc UnJoy                       ; OK!

        jsr CheckJump                   ; couldn't move... might be able to jump

UnJoy

        ;FALL THROUGH....
        ;------------------------------------------------------------------------------

CheckCompletion

        ; Check for completion of grid
        ; Switches player to locked when complete

        ; Make sure all cubes are stationary

        ldx #NUMBEROFCUBES-1
Still   lda CubeX,x
        bmi DeadS
        lda CubeYDir,x
        bpl NupNot                      ; its moving!
DeadS   dex
        bpl Still


        ; Looking good... everything is stationary
        ; but we must wait until all cubes are in their home spots!

        ldy #3
NextR   ldx #3

NextC   lda Grid,x
        and TargetBit,y
        beq NullSq                      ; no target there

        lda Grid,x
        and TargetOwnerBit,y
        cmp TargetOwnerBit,y
        bne NupNot
        
NullSq  dex
        bpl NextC
        dey
        bpl NextR        




        lda #MODE_LOCK+20
        sta PlayerCounter               ; lock for completion

NupNot
Ret     rts



;------------------------------------------------------------------------------

AIDelay

        cpx #0
        bne NotP0

        jsr Joystick
        bpl NotP0
        jmp ToNormal
        
NotP0
        lda #%1111
        jsr CountDown
        bcc ToNormal
        rts


;------------------------------------------------------------------------------





AIRiseUp

        ; Rise up out of a cube

        lda #%11 ;111
        jsr CountDown
        bcs StillRising
  
        lda #MODE_DELAY + 20
        sta PlayerCounter,x

        bne FixFlag


ToNormal

        lda #MODE_NORMAL
        sta PlayerCounter,x             ; switch to normal AI

FixFlag
        lda PlayerFlags,x
        and #255-FLAG_RISE
        sta PlayerFlags,x

        lda #0
        sta PlayerY,x

        jsr TieToCube

StillRising
        rts


AISink

        lda timer
        and #%11
        bne Sinker

        inc PlayerCounter,x

Sinker

        lda PlayerCounter,x
        and #%11111
        cmp #SPRITE_HEIGHT+1
        bcs Repos

        rts

Repos



        ; Lurking, waiting for a cube

        lda #MODE_LURK + SPRITE_HEIGHT
        sta PlayerCounter,x

        lda PlayerFlags,x
        ora #FLAG_RISE
        sta PlayerFlags,x

        rts


AILurk


        ; choose a new cube

        jsr Random
        cmp #30
        bcs Lurker
        lda rnd+1 ;jsr Random
        cmp #220
        bcc Lurker

 
        and #7
        cmp #NUMBEROFCUBES
        bcs Lurker

 
        tay
 
        lda CubeX,y
        bmi Lurker                      ; cube not present

        sty AIL_LurkerCube

        lda PlayerCube,x
        and #%11111000
        ora AIL_LurkerCube
        sta PlayerCube,x

        jsr TieToCube

        lda #MODE_RISE + SPRITE_HEIGHT
        sta PlayerCounter,x

        lda PlayerFlags,x
        ora #FLAG_RISE
        sta PlayerFlags,x

Lurker
LRet    rts



;------------------------------------------------------------------------------


MoveCube
        
        ; Erase all in a single swipe

        ldy #PFC-1
        lda #0
        ldx #128
ZapPF   sta playfield,y
        sta playfield+PFC,y
        stx playfield+PFC+PFC,y
        dey
        bpl ZapPF

        ; Move cubes and draw

        ldx #NUMBEROFCUBES
ShiftC2 dex
        bmi LRet                        ; any rts will do

        lda CubeX,x
        bmi ShiftC2                     ; non-cube
        ldy CubeYDir,x
        bpl TooFar1                     ; not moving (relay branch)

StopRelay

        jmp stopped


TooFar1

                ; This cube may be marked as slow-moving
                ; Typically this is auto-moved cubes.... here we check if its time to move 'em

        IF SLOW_AUTOMOVE

                lda CubeX,x
                and #MOVE_SLOW
                beq NotSlow
                lda timer
                and #MOVE_SLOW_MASK
                bne StopRelay
NotSlow
        ENDIF


        tya
        lsr
        lsr
        lsr
        lsr
        and #3
        tay                             ; 0-3 direction

        lda CubeX,x
        and #%11100000
        sta MC_temp
         
        clc
        lda CubeX,x
        adc DirMoveX,y
        and #31
        ora MC_temp
        sta CubeX,x

        lda CubeYDir,x
        and #$f0
        sta MC_DirOnly

        clc
        lda CubeYDir,x
        adc DirMoveY,y
        and #$0F
        ora MC_DirOnly
        sta CubeYDir,x


        lda DirMoveX,y
        asl
        asl
        sta MC_temp

        ldy #1
Shifter 
                ; Shift ALL players on this cube

        txa
        eor PlayerCube,y
        and #7
        bne not0

        clc
        lda MC_temp
        adc PlayerX,y
        sta PlayerX,y

not0    dey
        bpl Shifter

        lda CubeX,x
        and #31
        tay
        cpy #16
        bcs stopped

        lda Mod5,y
        bne stopped

        lda CubeYDir,x
        and #15
        tay
        lda Mod5,y
        bne stopped

                ; hit a gridlock boundary, so halt

        lda CubeYDir,x
        ora #$80
        sta CubeYDir,x                  ; halt movement

                ; Sound!

        ldy #SOUND_OFF
        jsr Sound

stopped
DrawCube

        ; Draw/Erase a single cube at x,y location

        ; pass
        ; X     = index of cube (preserved)
        ;       CUBE MUST EXIST (That is, no passing empty slots)

        ; the y position simply gives us the ROW (*5) of the start of draw
        ; the X position is mangled to give us byte information.
        ; TRICKY due to the odd layout of playfields

        ; use indexing into (predrawn) table lines to make it simple
        ; 5 bytes per x position = 


                ; Flash cubes (by skipping draw) when the tableaux has been completed

        lda Cubes
        and #CUBES_COMPLETED
        beq NDraw
        lda timer
        and #8
        beq SkipDr                      ; skip draw on tableaux completion

NDraw

        stx DC_saveX

        lda CubeX,x
        and #31
        tay
        cpy #16
        bcc NoBottom

                ; Draw the sliding cube on the bottom

        lda PreDrawn3,y
        sta TargetDisplay+8
        lda PreDrawn4,y
        sta TargetDisplay+9            ; this is pretty neato... just two bytes for the bottom sliding cube :))

NoBottom

                ; Draw in the grid area playfield

        lda CubeYDir,x
        and #15
        tax                             ; playfield location of start

        lda #3
        sta DC_CubeLine

DrawLoop

        lda PreDrawn0,y
        ora playfield,x
        sta playfield,x

        lda PreDrawn1,y
        ora playfield+PFC,x
        sta playfield+PFC,x

        lda PreDrawn2,y
        ora playfield+PFC+PFC,x
        sta playfield+PFC+PFC,x

        inx

        dec DC_CubeLine
        bpl DrawLoop

        ldx DC_saveX
SkipDr
        jmp ShiftC2


DirMoveY        .byte $fF       ;...continues...
DirMoveX        .byte 0,$01,0,$FF
DirMove4X       .byte 0,4,0,-4
DirMove5Y       .byte -CUBEXY
DirMove5X       .byte 0,CUBEXY,0,-CUBEXY


;------------------------------------------------------------------------------

;               PAL             NTSC
; 0000
; 0001          grey            light yellow
; 0010          orange          orange
; 0011          lime green      peach
; 0100          orange          pink
; 0101          blue green      purple
; 0110          pink red        dark blue
; 0111          aqua            blue
; 1000          purple          blue
; 1001          aqua            blue
; 1010          purple          aqua
; 1011          blue            green/blue
; 1100          blue purple     melon green
; 1101          dk.blue         olive
; 1110          grey            lt.green
; 1111          grey            orangy


PlayerColour

        IF COMPILE_VERSION = PAL


                IF PLAYER_WEARS_HAT
                        .byte %01100010
                        .byte %01100010
                ELSE
                        .byte %00011100
                        .byte %00011100 ; << hat  (TIMING PRECLUDES ACCURATE USE)
                ENDIF

                .byte %00011100
                .byte %00011100
                .byte %00011100
                .byte %00011100
                .byte %00011100
                .byte %00011100 ; << body

                .byte %11011011 ; << pants

                .byte %01100010
                .byte %01100010 ; << boots
        ELSE

                IF PLAYER_WEARS_HAT
                        .byte %01000010
                        .byte %01000010
                ELSE
                        .byte %00001100
                        .byte %00001100 ; << hat  (TIMING PRECLUDES ACCURATE USE)
                ENDIF

                .byte %00001100
                .byte %00001100
                .byte %00001100
                .byte %00001100
                .byte %00001100
                .byte %00001100

                .byte %10011010 ; << pants

                .byte %01000010
                .byte %01000010 ; << boots

        ENDIF



MTargetArea

        ; Y = scanline (actually, 1/2 scanline... index to sprite graphics)
        ; X = playfield index (0-19)
        ; X incremented by 1, Y incremented by 3


     ; LINE 1 -------------------------------

        sta WSYNC               ; 3                             wait for start of hblank

        lda #%00000000          ; 2
        sta PF0                 ; 3 ->8         <    (@5)
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->15        < 28 (@12)
        lda playfield+PFC,x     ; 4
        sta PF2                 ; 3 ->22        < 39 (@19)

                ; right pf

        lda playfield+PFC+PFC,x ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->31        < 49 (@28)

        stx DS_temp             ; 3

        lda TargetIndex,x       ; 3
        tax                     ; 2
        lda 0,x                 ; 4
        sta MTA_TargetL         ; 3
;        and MT_TargetMask       ; 3
 bit 0
        sta PF1                 ; 3 ->52        < 55 (@49)
        lda 1,x                 ; 4
        sta MTA_TargetR         ; 3
;        and MT_TargetMask       ; 3
 bit 0
        sta PF2                 ; 3 ->65        < 66 (@62)

        ldx DS_temp             ; 3 ->68

        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sec                     ; 2
        sbc SpriteEnd           ; 3
        adc #SPRITE_HEIGHT      ; 2                                     || NOMINAL LINE CHANGE
        bcs .Draw               ; 2(3)
        
        nop                     ; 2
        nop                     ; 2
        sec                     ; 2
        bcs .skipDraw           ; 3

.Draw
        lda (Sprite),y          ; 5
        sta PLAYER0SHAPE        ; 3

.skipDraw

        ;-->18 cycles used
        ;--------------------------

; = 86

     ; LINE 2 -------------------------------

                ; OVER BY 10

        lda #%10000000          ; 2
        sta PF0                 ; 3 -> 73
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->22        < 28 (@19)
        lda playfield+PFC,x     ; 4
        sta PF2                 ; 3 ->29        < 39 (@26)

        lda MTargetMaskInit,x   ; 4
        sta MT_TargetMask       ; 3

                ; right pf

        lda playfield+PFC+PFC,x ; 4
        sta PF0                 ; 3 ->36        < 49 (@33)

        lda MTA_TargetL         ; 3
        and MT_TargetMask       ; 3
        sta PF1                 ; 3 ->42        < 55 (@39)
        lda MTA_TargetR         ; 3
        and MT_TargetMask       ; 3
        sta PF2                 ; 3 ->48        < 66 (@45)


        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd+1         ; 3
        adc #SPRITE_HEIGHT      ; 2
        bcs .Draw2              ; 2(3)

        nop                     ; 2
        nop                     ; 2
        sec                     ; 2
        bcs .skipDraw2          ; 3

.Draw2
        lda (Sprite+2),y        ; 5
        sta PLAYER1SHAPE        ; 3

.skipDraw2

        ;-->18 cycles used
        ;--------------------------

;->76

     ; LINE 3 -------------------------------

        lda #0                  ; 2
        sta PF0                 ; 3 ->5              (@2)
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->12        < 28 (@9)
        lda playfield+PFC,x     ; 4
        sta PF2                 ; 3 ->19        < 39 (@16)

        iny                     ; 2                             NEXT LINE FOR SPRITES

;\\10
        bit 0
        nop
        nop
;//
                ; right pf

        lda playfield+PFC+PFC,x ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->40        < 49 (@37)

        lda MTA_TargetL         ; 3
;        and MT_TargetMask       ; 3
 bit 0
       sta PF1                 ; 3 ->49        < 55 (@46)
        lda MTA_TargetR         ; 3
;        and MT_TargetMask       ; 3
 bit 0
        sta PF2                 ; 3 ->58        < 66 (@55)


        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd           ; 3
        adc #SPRITE_HEIGHT      ; 2
        bcs .Draw3              ; 2(3)

        nop                     ; 2
        nop                     ; 2
        sec                     ; 2
        bcs .skipDraw3          ; 3

.Draw3
        lda (Sprite),y          ; 5
        sta PLAYER0SHAPE        ; 3

.skipDraw3

        ;-->18 cycles used
        ;--------------------------


     ; LINE 4 -------------------------------

        lda #%10000000          ; 2
        sta PF0                 ; 3
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->12        < 28 (@9)
        lda playfield+PFC,x     ; 4
        sta PF2                 ; 3 ->19        < 39 (@16)

;\\20
;        bit 0
;        bit 0
; nop
        nop
        jsr Ret
;//

                ; right pf

        lda playfield+PFC+PFC,x ; 4
        sta PF0                 ; 3 ->46        < 49 (@43)
        lda MTA_TargetL         ; 3
        and MT_TargetMask       ; 3
; bit 0
        sta PF1                 ; 3 ->52        < 55 (@49)
        lda MTA_TargetR         ; 3
        and MT_TargetMask       ; 3
; bit 0
        sta PF2                 ; 3 ->58        < 66 (@55)


        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd+1         ; 3
        adc #SPRITE_HEIGHT      ; 2
        bcs .Draw4              ; 2(3)

        nop
        nop
        sec                     ; 2
        bcs .skipDraw4          ; 3

.Draw4
        lda (Sprite+2),y        ; 5
        sta PLAYER1SHAPE        ; 3

.skipDraw4

        ;-->18 cycles used
        ;--------------------------


     ; LINE 5 -------------------------------

        lda #%00000000          ; 2
        sta PF0                 ; 3 ->5         <    (@2)
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->12        < 28 (@9)
        lda playfield+PFC,x     ; 4
        sta PF2                 ; 3 ->19        < 39 (@16)

        iny                     ; 2

        inx

                ; right pf

        lda playfield+PFC+PFC-1,x ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->30        < 49 (@27)
        lda MTA_TargetL         ; 3
;        and MT_TargetMask       ; 3
  bit 0
       sta PF1                 ; 3 ->39        < 55 (@36)
        lda MTA_TargetR         ; 3
;        and MT_TargetMask       ; 3
 bit 0
        sta PF2                 ; 3 ->48        < 66 (@45)

        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd           ; 3
        adc #SPRITE_HEIGHT      ; 2
        bcs .Draw5              ; 2(3)

        nop
        nop
        sec                     ; 2
        bcs .skipDraw5          ; 3

.Draw5
        lda (Sprite),y          ; 5
        sta PLAYER0SHAPE        ; 3

.skipDraw5

        ;-->18 cycles used
        ;--------------------------

;->64
        lda #%10000000          ; 2 ->0         <    (@0)
        sta PF0                 ; 3

;->76

     ; LINE 6 -------------------------------


        lda playfield-1,x         ; 4
        sta PF1                 ; 3 ->10        < 28 (@7)
        lda playfield+PFC-1,x     ; 4
        sta PF2                 ; 3 ->17        < 39 (@14)

        ;--------------------------
        ; SPRITE DRAW

        tya                     ; 2
        sbc SpriteEnd+1         ; 3
        adc #SPRITE_HEIGHT      ; 2
        bcs .Draw6              ; 2(3)

        nop                     ; 2
        nop                     ; 2
        sec                     ; 2
        bcs .skipDraw6          ; 3

.Draw6
        lda (Sprite+2),y        ; 5
        sta PLAYER1SHAPE        ; 3

.skipDraw6

        ;-->18 cycles used
        ;--------------------------

; ->35
                ; right pf

        lda playfield+PFC+PFC-1,x ; 4
        sta PF0                 ; 3 ->42        < 49 (@39)
        lda MTA_TargetL         ; 3
        and MT_TargetMask
        sta PF1                 ; 3 ->48        < 55 (@45)
        lda MTA_TargetR         ; 3
        and MT_TargetMask
        sta PF2                 ; 3 ->54        < 66 (@51)

        iny                     ; 2

        rts                     ; 6 ->62


;------------------------------------------------------------------------------


;------------------------------------------------------------------------------


SetupBottomScore

        ; Display lives/level if joystick button pressed


        lda #0
        sta S2
        sta S3

LivesLevel

        ldy #11                 ; #icon
        sty S0
        iny                     ; &icon
        sty S4

        lda level
        lsr
        lsr
        lsr
        lsr
        lsr
        clc
        adc #1
        sta S5                  ; lives

        lda level
        and #%11111
        cmp #10
        bcs sub10
        adc #1
        sta S1
        bcc ldone
sub10   sbc #10
        inc S1
        bcs sub10
        adc #11
        sta S2
ldone

        sta WSYNC
        rts


PSI     .byte 8,4,0

SetupTopScore

        ; Place appropriate data in score display line

        ldx #2
inscore
        ldy PSI,x
        
        lda PlayerScore,x
        and #15
        sta S0+2,y

        lda PlayerScore,x
        lsr
        lsr
        lsr
        lsr
        sta S0,y

        dex
        bpl inscore

 
                 ; Now remove leading zeros

        inx
        txa                     ; now 0
rlz     ora S0,x
        beq nlz
        inc S0,x
nlz     inx
        inx
        cpx #10
        bcc rlz

        inc S0+10

        rts

;------------------------------------------------------------------------------

AddScore

        ; CANNOT USE ANY OVERLAY VARIABLES --- 

        ; add (a,y) to player score
        ; a = low, y = high

        sed
        clc
        adc PlayerScore
        sta PlayerScore
        tya
        adc PlayerScore+1
        sta PlayerScore+1
        lda #0
        adc PlayerScore+2
        sta PlayerScore+2

;        bcc nWrap

;        lda #$99
;        sta PlayerScore
;        sta PlayerScore+1
;        sta PlayerScore+2

;nWrap

        cld

                ; flash score

;        lda Cubes
;        ora #%00001000
;        sta Cubes


        rts

;------------------------------------------------------------------------------

;SubScore

        ; subtract (x,y) from player score
        ; x = low, y = high

;        sed
;        sec
;        lda PlayerScore
;        stx PlayerScore
;        sbc PlayerScore
;        sta PlayerScore
;        lda PlayerScore+1
;        sty PlayerScore+1
;        sbc PlayerScore+1
;        lda PlayerScore+2
;        sbc #0
;        sta PlayerScore+2
;        cld

;        bcs SubOK

;        lda #0
;        sta PlayerScore
;        sta PlayerScore+1
;        sta PlayerScore+2               ; wrapped to 0

;SubOK   rts

;------------------------------------------------------------------------------

        org $FF00

ScoreLine

        ; WARNING:
        ; DIGIT DATA CANNOT CROSS PAGE BOUNDARY!  NEITHER CAN THE INNER LOOP!

        ; pass the 6 'digits' in S0-S5 inclusive
        ; mangled on return


        tsx
        stx stkp

        IF ATTRACT_MODE
                eor BackColour
        ENDIF

        sta SL_temp
        eor #7
        sta COLUP0
        sta COLUP1

;        lda #D0                 ; reflect / score
;        sta CTRLPF
        ldy #0
        sty VDELP0
        sty REFP0
        sty REFP1

                ; Convert the digits S0-S5 to pointers to graphics

initscore
        
        lda S0,y
        asl
        asl
        asl
        adc #<DigitNull
        sta S0,y
        lda #>DigitNull
        adc #0                 ; << ITS A GIVEN THAT DIGITS DON'T CROSS PAGE
        sta S0+1,y

        iny
        iny
        cpy #12
        bcc initscore
 


        lda #3
        sta NUSIZ0
        sta NUSIZ1              ; 3 copies close

        sta WSYNC

        jsr Ret
        jsr Ret
        inc loop
        dec.w loop

        sta RESP0

        sta WSYNC

        jsr Ret
        jsr Ret
        jsr Ret

        sta.w RESP1
           
        sta HMCLR
        lda #%10100000                  ; "2 more than below"?
        sta HMP1
        lda #%11000000                  ;\-->
        sta HMP0
        sta WSYNC
        sta HMOVE

        sta WSYNC


        ldx #11
pause1  dex
        bne pause1
        nop
        nop


        ; Tomas Jentzsch magic...

        ldy #7
mainloop

        sty loop                ; 3

 nop ;2
 nop ;2

        lda (S0),y ;5
        sta PLAYER0SHAPE                ; 3 = 15

 bit 0 ;3

        lda (S1),y              ; 5
        sta PLAYER1SHAPE                ; 3

        IF COMPILE_ILLEGALOPCODES
        .byte $b3,S5            ; 5 lax (S5),y ;5        illegal opcode
        ELSE
        lda (S5),y
        ENDIF

        txs                     ; 2
        lda (S2),y              ; 5
        sta sreg                ; 3
        
        IF COMPILE_ILLEGALOPCODES
        .byte $b3,S3            ; 5 lax (S3),y ;5        illegal opcode
        ELSE
        lda (S3),y
        ENDIF

        lda (S4),y              ; 5 = 33

        ldy sreg                ; 3
        sty PLAYER0SHAPE                ; 3
        stx PLAYER1SHAPE                ; 3
        sta PLAYER0SHAPE                ; 3
        tsx                     ; 2
        stx PLAYER1SHAPE                ; 3 = 17

        ;//15 cycles start

        lda loop                ; 3
        tay                     ; 2
        eor SL_temp             ; 3
        sta COLUP0              ; 3
        sta.w COLUP1            ; 4

        ;\\end

        lda (S0),y ;5
        sta PLAYER0SHAPE                ; 3 = 15

  bit 0 ;3

        lda (S1),y              ; 5
        sta PLAYER1SHAPE                ; 3

        IF COMPILE_ILLEGALOPCODES
        .byte $b3,S5            ; 5 lax (S5),y ;5        illegal opcode
        ELSE
        lda (S5),y
        ENDIF

        txs                     ; 2
        lda (S2),y              ; 5
        sta sreg                ; 3

        IF COMPILE_ILLEGALOPCODES
        .byte $b3,S3            ; 5 lax (S3),y ;5        illegal opcode
        ELSE
        lda (S3),y
        ENDIF

        lda (S4),y              ; 5 = 33

        ldy sreg                ; 3
        sty PLAYER0SHAPE                ; 3
        stx PLAYER1SHAPE                ; 3
        sta PLAYER0SHAPE                ; 3
        tsx                     ; 2
        stx PLAYER1SHAPE                ; 3 = 17

        ldy loop
        dey
        bpl mainloop            ; 3 =  8

        ldx stkp
        txs

        lda #0
;        sta PF2
        sta PLAYER0SHAPE
        sta PLAYER1SHAPE

        rts


;------------------------------------------------------------------------------


    org $FFFC
    .word start                 ; irq
    .word start                 ; reset vector

;EOF
